import supertest from 'supertest';
import db from '../../database/connection';
import Recipe from '../../models/recipe';
import app from '../../server';

const request = supertest(app);

beforeAll(async () => {
  await db.migrate.latest();
  await db.seed.run();
});

describe('Getting recipes - Endpoint', () => {
  it('Should get all recipes', async () => {
    const response = await request.get('/recipes');

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);
    expect(response.body).toHaveProperty('message', 'OK');
    expect(response.body).toHaveProperty('data');
  });

  it('Should get only active recipes', async () => {
    const response = await request.get('/recipes');

    const recipes: Recipe[] = response.body.data;

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);

    recipes.forEach((recipe) => {
      expect(recipe.active).toBe(1);
    });
  });

  it('Should not get inactive recipes', async () => {
    const response = await request.get('/recipes');

    const recipes: Recipe[] = response.body.data;

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);

    recipes.forEach((recipe) => {
      expect(recipe.active).not.toBe(0);
    });
  });

  it('Should have recipe_id property', async () => {
    const response = await request.get('/recipes');

    const recipes: Recipe[] = response.body.data;

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);

    recipes.forEach((recipe) => {
      expect(recipe).toHaveProperty('recipe_id');
    });
  });

  it('Like number property should be zero or greather', async () => {
    const response = await request.get('/recipes');

    const recipes: Recipe[] = response.body.data;

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);

    recipes.forEach((recipe) => {
      expect(recipe.likes_number).toBeGreaterThanOrEqual(0);
    });
  });

  it('Should have array of ingredients', async () => {
    const response = await request.get('/recipes');

    const recipes: Recipe[] = response.body.data;

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);

    recipes.forEach((recipe) => {
      expect(Array.isArray(recipe.ingredients)).toBeTruthy();
    });
  });

  it('Each recipe should have a tags property', async () => {
    const response = await request.get('/recipes');

    const recipes: Recipe[] = response.body.data;

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);

    recipes.forEach((recipe) => {
      expect(recipe).toHaveProperty('tags');
    });
  });

  it('Each recipe should have a steps property', async () => {
    const response = await request.get('/recipes');

    const recipes: Recipe[] = response.body.data;

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);

    recipes.forEach((recipe) => {
      expect(recipe).toHaveProperty('steps');
    });
  });
  it('recipe_id should be the same as the passed id', async () => {
    const response = await request.get('/recipes/1');

    const recipe: Recipe = response.body.data;

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);

    expect(recipe.recipe_id).toBe(1);
  });
  it('recipe should have the property recipe_id', async () => {
    const response = await request.get('/recipes/1');

    const recipe: Recipe = response.body.data;

    expect(response.status).toBe(200);
    expect(response.body).not.toBe(null);

    expect(recipe).toHaveProperty('recipe_id');
  });
  it('should not return a recipe with an id that doesnt exists', async () => {
    const response = await request.get('/recipes/9999');
    expect(response.status).toBe(404);
    //expect(response.body).toBe(null);
  });
});

describe('Creating recipes - Endpoint', () => {
  it('Should create a recipe', async () => {
    const response = await request
      .post('/recipes')
      .send({
        ingredients: [
          'farinha',
          'água',
          'azeite',
        ],
        description: 'Uma pizza bem bacana',
        preparation_time: 60,
        difficulty: 5,
        portions: 8,
        user_id: '241874381247udasgfd',
        status: 'OK',
        title: 'Pizza',
        tags: [
          { tag_id: 1 },
        ],
        steps: [
          {
            title: 'Passo 1',
            description: 'Desc',
            position: 0,
          },
        ],
      });
    expect(response.status).toBe(201);
    expect(response.body).not.toBe(null);
    expect(response.body).toHaveProperty('message', 'Recipe created');
    expect(response.body).toHaveProperty('data');
  });

  it('Should not create a recipe with missing required fields', async () => {
    const response = await request
      .post('/recipes')
      .send({
        ingredients: [
          'farinha',
          'água',
          'azeite',
        ],
        description: 'Uma pizza bem bacana',
        preparation_time: 60,
        difficulty: 5,
        status: 'OK',
        tags: [
          { tag_id: 1 },
        ],
        steps: [
          {
            title: 'Passo 1',
            description: 'Desc',
            position: 0,
          },
        ],
      });
    expect(response.status).toBe(400);
  });

  it('Should not create a recipe with invalid tag', async () => {
    const response = await request
      .post('/recipes')
      .send({
        ingredients: [
          'farinha',
          'água',
          'azeite',
        ],
        description: 'Uma pizza bem bacana',
        preparation_time: 60,
        difficulty: 5,
        user_id: '241874381247udasgfd',
        status: 'OK',
        portions: 6,
        title: 'Pizza',
        tags: [
          { tag_id: 666 },
        ],
        steps: [
          {
            title: 'Passo 1',
            description: 'Desc',
            position: 0,
          },
        ],
      });
    expect(response.status).not.toBe(201);
  });
});

afterAll(async () => {
  await db.migrate.rollback();
  await db.destroy();
});
