import db from '../../database/connection';
import Recipe from '../../models/recipe';
import { createRecipe, getAllRecipes, getRecipeById } from '../../services/recipeService';

beforeAll(async () => {
  await db.migrate.latest();
  await db.seed.run();
});

describe('Getting all recipes - Service', () => {
  it('Should get all recipes', async () => {
    const recipes = await getAllRecipes({});

    expect(recipes).not.toBe(null);
  });

  it('Should get only active recipes', async () => {
    const recipes = await getAllRecipes({});

    recipes.forEach((recipe) => {
      expect(recipe.active).toBe(1);
    });
  });

  it('Should not get inactive recipes', async () => {
    const recipes = await getAllRecipes({});

    recipes.forEach((recipe) => {
      expect(recipe.active).not.toBe(0);
    });
  });

  it('Should have recipe_id property', async () => {
    const recipes = await getAllRecipes({});

    recipes.forEach((recipe) => {
      expect(recipe.recipe_id).not.toBe(null);
    });
  });

  it('Like number property should be zero or greather', async () => {
    const recipes = await getAllRecipes({});

    recipes.forEach((recipe) => {
      expect(recipe.likes_number).toBeGreaterThanOrEqual(0);
    });
  });

  it('Should have array of ingredients', async () => {
    const recipes = await getAllRecipes({});

    recipes.forEach((recipe) => {
      expect(Array.isArray(recipe.ingredients)).toBeTruthy();
    });
  });

  it('Each recipe should have a tags property', async () => {
    const recipes = await getAllRecipes({});

    recipes.forEach((recipe) => {
      expect(recipe).toHaveProperty('tags');
    });
  });

  it('Each recipe should have a steps property', async () => {
    const recipes = await getAllRecipes({});

    recipes.forEach((recipe) => {
      expect(recipe).toHaveProperty('steps');
    });
  });
});

describe('Getting recipes by id - Service', () => {
  it('Should get a recipe by specific id', async () => {
    const [recipe] = await getRecipeById({ recipe_id: 1 });
    expect(recipe).not.toBe(undefined);
  });
  it('Should not get a recipe by inexistent id', async () => {
    const [recipe] = await getRecipeById({ recipe_id: 9999 });
    expect(recipe).toBe(undefined);
  });
  it('Should get only active recipe', async () => {
    const [recipe] = await getRecipeById({ recipe_id: 1 });
    expect(recipe.active).toBe(1);
  });
  it('Should not get inactive recipe', async () => {
    const [recipe] = await getRecipeById({ recipe_id: 1 });
    expect(recipe.active).not.toBe(0);
  });
  it('Should return a recipe with the id specified', async () => {
    const [recipe] = await getRecipeById({ recipe_id: 1 });
    expect(recipe.recipe_id).toBe(1);
  });
  it('Should return only 1 recipe if the id exists', async () => {
    const recipe = await getRecipeById({ recipe_id: 1 });
    expect(recipe.length).toBe(1);
  });
});

describe('Creating recipes', () => {
  it('Should insert Recipe without errors', async () => {
    const recipe = new Recipe(
      'Pizza',
      'Uma pizza bem bacana',
      'Uma colher de óleo; Farinha; Queijo; Tomate',
      60,
      5,
      6,
      '241874381247udasgfd',
      'OK',
      [
        { tag_id: 1 }, { tag_id: 2 },
      ],
      [
        { title: 'Passo 1', description: 'Descrição passo 1', position: 0 },
        { title: 'Passo 2', description: 'Descrição passo 2', position: 1 },
      ],
    );
    expect(async () => createRecipe(recipe)).not.toThrowError();
  });

  it('Should return created recipe ID', async () => {
    const recipe = new Recipe(
      'Feijoada',
      'Feijoada completa',
      'Uma colher de óleo; Feijões; Carne; Bacon; Louro',
      120,
      7,
      12,
      '241874381247udasgfd',
      'OK',
      [
        { tag_id: 3 }, { tag_id: 5 },
      ],
      [
        { title: 'Passo 1', description: 'Descrição passo 1', position: 0 },
        { title: 'Passo 2', description: 'Descrição passo 2', position: 1 },
      ],
    );
    const recipeId = await createRecipe(recipe);
    expect(recipeId).toBeGreaterThan(0);
  });
});

afterAll(async () => {
  await db.migrate.rollback();
  await db.destroy();
});
