import db from '../database/connection';
import Recipe from '../models/recipe';
import Tag from '../models/tag';

interface InsertRecipeId{
  recipe_id: number
}

export async function saveRecipe(recipe: Recipe): Promise<number> {
  const {
    ingredients, description, preparation_time, difficulty,
    user_id, status, likes_number, title, steps, portions,
  } = recipe;
  const { tags } = recipe;
  return db.transaction(async (tr) => {
    const [insertedRecipe]: InsertRecipeId[] = await db('recipes').transacting(tr).insert({
      ingredients,
      description,
      preparation_time,
      difficulty,
      user_id,
      status,
      likes_number,
      title,
      portions,
    }, ['recipe_id']);
    const recipesTags = tags.map((t) => (
      { tag_id: t.tag_id, recipe_id: insertedRecipe.recipe_id }
    ));
    const recipeSteps = steps.map((s) => (
      { ...s, recipe_id: insertedRecipe.recipe_id }
    ));
    await db('steps').transacting(tr).insert(recipeSteps);
    await db('recipes_tags').transacting(tr).insert(recipesTags);
    return insertedRecipe.recipe_id;
  });
}

/**
 * Returns all recipes
 * @returns Promise<Recipe[]>
 */
export async function findAllRecipes(): Promise<Recipe[]> {
  const recipes = await db.select('*').from<Recipe>('recipes')
    .where('active', 1);

  return populateRecipesWithTagsAndSteps(recipes);
}

export async function findRecipesByQueryParams(
  title: string,
  tags: number[],
): Promise<Recipe[]> {
  const isTagsEmpty: boolean = tags.length === 0;
  const isTitleEmpty: boolean = typeof title === 'undefined';

  const recipes = await db.select('*')
    .from<Recipe>('recipes')
    .where('active', 1)
    .andWhere((qb) => {
      if (!isTitleEmpty && isTagsEmpty) {
        title = `%${title.toLowerCase()}%`;
        qb.whereRaw('LOWER(title) like ?', [title]);
      }
      if (isTitleEmpty && !isTagsEmpty) {
        qb.whereIn('recipe_id', db.select('recipe_id').from('recipes_tags').whereRaw(`tag_id in (${tags.map((_) => '?').join(',')})`, [...tags]));
      }
      if (!isTitleEmpty && !isTagsEmpty) {
        title = `%${title.toLowerCase()}%`;
        qb.whereIn('recipe_id', db.select('recipe_id').from('recipes_tags').whereRaw(`tag_id in (${tags.map((_) => '?').join(',')})`, [...tags]));
        qb.whereRaw('LOWER(title) like ?', [title]);
      } else {
        qb.whereRaw('1 = 1');
      }
    });
  return populateRecipesWithTagsAndSteps(recipes);
}

/**
 * Receives an user_id
 * @param user
 * @returns Promise<Recipe[]>
 */
export async function findRecipesByUser(
  user: string,
): Promise<Recipe[]> {
  const recipes = await db.select('*')
    .from<Recipe>('recipes')
    .where('active', 1)
    .andWhere('user_id', user);
  return populateRecipesWithTagsAndSteps(recipes);
}

/**
 * Receives a recipe_id
 * @param recipe_id
 * @returns Promise<Recipe[]>
 */
export async function findRecipeById(
  recipe_id: number,
): Promise<Recipe[]> {
  const recipe = await db.select('*')
    .from<Recipe>('recipes')
    .where('active', 1)
    .andWhere('recipe_id', recipe_id);
  return populateRecipesWithTagsAndSteps(recipe);
}

/**
 * Receives a list of recipe_id
 * @param recipes_id
 * @returns Promise<Recipe[]>
 */
export async function findRecipesByListOfId(
    recipesId: number[],
): Promise<Recipe[]> {
  const recipe = await db.select('*')
      .from<Recipe>('recipes')
      .whereIn('recipe_id', recipesId)
      .andWhere('active', 1);
  return populateRecipesWithTagsAndSteps(recipe);
}

/**
 * Receives an array of recipes and adds its tags
 * @param recipes
 * @returns Promise<Recipe[]>
 */
async function populateRecipesWithTagsAndSteps(recipes: Recipe[]): Promise<Recipe[]> {
  const result = recipes.map(async (recipe) => {
    if (!recipe.recipe_id) throw new Error('Recipe recipe_id cannot be empty ');
    const tags = await db.select('*').from<Tag>('tags')
      .whereIn('tag_id', db.select('tag_id').from('recipes_tags').whereRaw('recipe_id = ?', [recipe.recipe_id]));
    const steps = await db.select('*').from('steps')
      .where('recipe_id', recipe.recipe_id);
    const newRecipe = recipe;
    newRecipe.tags = tags;
    newRecipe.steps = steps;
    return newRecipe;
  });

  return Promise.all(result);
}
